import hashlib
from datetime import datetime

from lxml import etree

from django.conf import settings
from django.core.files.base import ContentFile

from banzai.models import Package


class MailPackage(object):

    def __init__(self, email_from, name_from, subject, message, send_at=None,
                 headers=[], attach_images=u'0', description=u''):
        self.email_from = email_from
        self.name_from = name_from
        self.subject = subject
        self.message = message
        self.send_at = send_at
        self.headers = headers
        self.attach_images = attach_images
        self.description = description

        self._recipients_data = []
        self._xml = etree.Element('list')
        self._generation_complete = False
        self._package = None

    def add_recipients(self, recipients_list):
        if isinstance(recipients_list, dict):
            recipients_list = [recipients_list]
        for recipient in recipients_list:
            if isinstance(recipient, dict):
                self.add_recipient(
                    recipient['email_to'],
                    recipient.get('name_to', u''),
                    recipient.get('header', {}),
                    recipient.get('fields', {})
                )
            elif isinstance(recipient, (str, unicode)):
                self.add_recipient(recipient)
            else:
                raise RuntimeError('recipients_list must be a dict or string!')

    def add_recipient(self, email_to, name_to=u'', header={}, fields={}):
        if self._generation_complete:
            raise RuntimeError('Impossibly add recipient, because '
                               'generation already complete!')
        self._recipients_data.append({
            'email_to': email_to,
            'name_to': name_to,
            'header': header,
            'fields': fields
        })

    def save(self):
        if self._package is None:
            file_name = '{0}#{1}'.format(datetime.now(), settings.SECRET_KEY)
            file_name = hashlib.md5(file_name).hexdigest()
            file_name = '{0}.xml'.format(file_name)

            package_obj = Package(
                emails_all=len(self._recipients_data),
                description=self.description
            )
            xml_content = ContentFile(self.xml_tostring())
            package_obj.file.save(file_name, xml_content)
            package_obj.save()
            self._package = package_obj
        return self._package

    def xml_tostring(self):
        self._generate()
        return etree.tostring(self._xml, encoding='UTF-8',
                              xml_declaration=True)

    def _generate(self):
        if self._generation_complete:
            return
        body_tag = etree.Element('body')

        data_tag = etree.Element('Data')
        data_tag.text = etree.CDATA(self.message)
        body_tag.append(data_tag)
        del data_tag

        email_from_tag = etree.Element('EmailFrom')
        email_from_tag.text = etree.CDATA(self.email_from)
        body_tag.append(email_from_tag)
        del email_from_tag

        name_from_tag = etree.Element('NameFrom')
        name_from_tag.text = etree.CDATA(self.name_from)
        body_tag.append(name_from_tag)
        del name_from_tag

        subject_tag = etree.Element('Subject')
        subject_tag.text = etree.CDATA(self.subject)
        body_tag.append(subject_tag)
        del subject_tag

        attach_images_tag = etree.Element('AttachImages')
        attach_images_tag.text = etree.CDATA(self.attach_images)
        body_tag.append(attach_images_tag)
        del attach_images_tag

        if self.send_at is not None:
            send_at_tag = etree.Element('SendAt')
            send_at_str = self.send_at.strftime('%Y/%m/%d %H:%M')
            send_at_tag.text = etree.CDATA(send_at_str)
            body_tag.append(send_at_tag)
            del send_at_tag, send_at_str

        for header in self.headers:
            header_tag = etree.Element('Header')
            header_tag.attrib['name'] = header['name']
            header_tag.attrib['value'] = header['value']
            body_tag.append(header_tag)
            del header_tag

        self._xml.append(body_tag)
        del body_tag

        users_tag = etree.Element('users')
        for recipient in self._recipients_data:
            current_user_tag = etree.Element('user')

            email_to_tag = etree.Element('EmailTo')
            email_to_tag.text = etree.CDATA(recipient['email_to'])
            current_user_tag.append(email_to_tag)
            del email_to_tag

            if recipient['name_to']:
                name_to_tag = etree.Element('NameTo')
                name_to_tag.text = etree.CDATA(recipient['name_to'])
                current_user_tag.append(name_to_tag)
                del name_to_tag

            if recipient['header']:
                header_tag = etree.Element('Header')
                header_tag.attrib['name'] = recipient['header']['name']
                header_tag.attrib['value'] = recipient['header']['value']
                current_user_tag.append(header_tag)
                del header_tag

            for field in recipient['fields']:
                field_tag = etree.Element(field['name'])
                field_tag.text = etree.CDATA(field['value'])
                current_user_tag.append(field_tag)
                del field_tag

            users_tag.append(current_user_tag)
            del current_user_tag

        self._xml.append(users_tag)
        del users_tag

        self._generation_complete = True



import requests
from lxml import etree

from django.conf import settings
from django.contrib.sites.models import Site

from banzai.models import Report, ReportFBL
from banzai.settings import (BANZAI_API_DOMAIN, BANZAI_API_VERSION,
                             BANZAI_API_KEY)


class BanzaiAPI(object):

    def __init__(self, package_obj):
        self._package = package_obj
        self._package_url = None

        self._api_key = BANZAI_API_KEY
        self._base_api_url = 'http://{0}/api/{1}/'.format(BANZAI_API_DOMAIN,
                                                          BANZAI_API_VERSION)

    @property
    def package_url(self):
        if self._package_url is None:
            current_site = Site.objects.get_current()
            url = 'http://{0}{1}{2}'.format(
                current_site.domain,
                settings.MEDIA_URL,
                self._package.file.url
            )
            self._package_url = url
        return self._package_url

    def _api_url(self, method):
        return '{0}{1}'.format(self._base_api_url, method)

    def _parse_xml(self, data):
        parser = etree.XMLParser()
        parser.feed(data)
        return parser.close()

    def add(self):
        get_params = {'key': self._api_key, 'url': self.package_url}
        req = requests.get(self._api_url('add_package'), params=get_params)

        elem = self._parse_xml(req.content)

        pack_id = elem.find('pack_id')
        if pack_id is not None:
            self._package.pack_id = pack_id.text

        status = elem.find('status')
        if status is not None:
            self._package.status = status.text

        self._package.save()
        return self._package

    def fast_add(self):
        get_params = {'key': self._api_key}
        post_data = {
            'package': self._package.file.read(),
            'description': self._package.description
        }
        req = requests.post(self._api_url('fast_add_package'),
                            params=get_params, data=post_data)

        elem = self._parse_xml(req.content)

        pack_id = elem.find('pack_id')
        if pack_id is not None:
            self._package.pack_id = pack_id.text

        status = elem.find('status')
        if status is not None:
            self._package.status = status.text

        user_count = elem.find('user_count')
        if user_count is not None:
            self._package.emails_correct = user_count.text

        self._package.save()
        return self._package

    def check(self):
        get_params = {'key': self._api_key}
        post_data = {
            'package': self._package.file.read(),
        }
        req = requests.post(self._api_url('check_package'),
                            params=get_params, data=post_data)

        elem = self._parse_xml(req.content)

        status = elem.find('status')
        if status is not None:
            self._package.status = status.text

        user_count = elem.find('user_count')
        if user_count is not None:
            self._package.emails_correct = user_count.text

        self._package.save()
        return {
            'status': self._package.status,
            'emails_correct': self._package.emails_correct
        }

    def status(self):
        get_params = {'key': self._api_key, 'pack_id': self._package.pack_id}
        req = requests.get(self._api_url('package_status'), params=get_params)

        elem = self._parse_xml(req.content)

        status = elem.find('status')
        if status is not None:
            self._package.status = status.text

        self._package.save()
        return self._package.status

    def report(self):
        get_params = {'key': self._api_key, 'pack_id': self._package.pack_id}
        req = requests.get(self._api_url('package_report'), params=get_params)

        elem = self._parse_xml(req.content)

        items = elem.findall('item')
        for item in items:
            email = ''
            reject_code = ''
            reject_message = ''

            email_elem = item.find('email')
            if email_elem is not None:
                email = email_elem.text

            code_elem = item.find('code')
            if code_elem is not None:
                reject_code = code_elem.text

            message_elem = item.find('message')
            if message_elem is not None:
                reject_message = message_elem.text

            report_obj, _ = Report.objects.get_or_create(
                package=self._package,
                email=email,
                reject_code=reject_code,
                reject_message=reject_message,
            )
        else:
            status = elem.find('status')
            if status is not None:
                report_obj, _ = Report.objects.get_or_create(
                    package=self._package,
                    status=status.text
                )
        return self._package.reports.all()

    def report_fbl(self):
        get_params = {'key': self._api_key, 'pack_id': self._package.pack_id}
        req = requests.get(self._api_url('package_report_fbl'),
                           params=get_params)

        elem = self._parse_xml(req.content)

        items = elem.findall('item')
        for item in items:
            email = ''

            email_elem = item.find('email')
            if email_elem is not None:
                email = email_elem.text

            report_obj, _ = ReportFBL.objects.get_or_create(
                package=self._package,
                email=email,
            )
        else:
            status = elem.find('status')
            if status is not None:
                report_obj, _ = ReportFBL.objects.get_or_create(
                    package=self._package,
                    status=status.text
                )
        return self._package.reports_fbl.all()

    def stop(self):
        pass



# coding: utf-8

from django.contrib import admin

from banzai.models import Package, Report, ReportFBL


class ReportInline(admin.StackedInline):

    extra = 0
    model = Report
    verbose_name = u'Отчёт'
    readonly_fields = ('status', 'email', 'reject_code', 'reject_message',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class ReportFBLInline(admin.StackedInline):

    extra = 0
    model = ReportFBL
    verbose_name = u'Отчёт о FBL'
    readonly_fields = ('status', 'email',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class PackageAdmin(admin.ModelAdmin):

    list_display = ('id', 'description', 'status', 'pack_id', 'emails_all',
                    'emails_correct',)
    readonly_fields = ('file', 'status', 'pack_id', 'emails_all',
                       'emails_correct', 'description',)
    inlines = (ReportInline, ReportFBLInline,)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


admin.site.register(Package, PackageAdmin)



from django.conf import settings


BANZAI_API_DOMAIN = getattr(settings, 'BANZAI_API_DOMAIN', 'api.get-n-post.ru')
BANZAI_API_VERSION = getattr(settings, 'BANZAI_API_VERSION', 'v1')
BANZAI_API_KEY = getattr(settings, 'BANZAI_API_KEY', '')
BANZAI_API_FAST_ADD_METHOD_RECIPIENTS_MAX_COUNT = getattr(
    settings, 'BANZAI_API_FAST_ADD_METHOD_RECIPIENTS_MAX_COUNT', 500)



from django.contrib.sites.models import Site

from banzai.mail import MailPackage
from banzai.api import BanzaiAPI
from banzai.settings import BANZAI_API_FAST_ADD_METHOD_RECIPIENTS_MAX_COUNT


def send_mail(subject, message, from_email, recipient_list, **kwargs):
    current_site = Site.objects.get_current()

    send_at = kwargs.pop('send_at', None)
    headers = kwargs.pop('headers', [])
    attach_images = kwargs.pop('attach_images', u'0')
    description = kwargs.pop('description', u'')

    mail_package = MailPackage(from_email, current_site.name, subject, message,
                               send_at, headers, attach_images, description)
    mail_package.add_recipients(recipient_list)
    package = mail_package.save()

    banzai_api_obj = BanzaiAPI(package)
    if package.emails_all < BANZAI_API_FAST_ADD_METHOD_RECIPIENTS_MAX_COUNT:
        package = banzai_api_obj.fast_add()
        sended_emails = package.emails_correct
    else:
        package = banzai_api_obj.add()
        sended_emails = package.emails_all

    return sended_emails



# coding: utf-8

from django.db import models


STATUS_GOOD_CODES = (
    ('S001', u'Пакет не был найден по указанному URL'),
    ('S002', u'Пакет загружен на сервер'),
    ('S004', u'Пакет рассылается'),
    ('S005', u'Пакет отправлен, ожидаем удаление информации о '
             u'пакете из системы'),
    ('S006', u'Во время отправки произошли ошибки'),
    ('S007', u'Некорректный формат пакета'),
    ('S008', u'Превышено время ожидания ответа сервера'),
    ('S009', u'Пакет получен'),
    ('S010', u'Пакет проверяется'),
    ('S011', u'Пакет обрабатывается (формируются сообщения)'),
    ('S012', u'Пакет поставлен в очередь на удаление'),
    ('S013', u'Пакет удален. Рассылка сообщений прекращена'),
    ('S015', u'Количество адресатов превышает установленный лимит'),
    ('S018', u'Пакет отправлен. Информация о пакете удалена'),
    ('S019', u'Пакет проверен, будет разослан в указанное время'),
)
STATUS_ERROR_CODES = (
    ('E000', u'Секретный ключ, указанный в параметрах операции, не опознан'),
    ('E001', u'Не передан URL'),
    ('E002', u'Не передан pack_id'),
    ('E003', u'Информация о пакете не найдена'),
    ('E004', u'Недостаточно трафика'),
    ('E005', u'Не оплачен тариф'),
    ('E006', u'Некорректные данные в пакете'),
)
STATUS_CODES = STATUS_GOOD_CODES + STATUS_ERROR_CODES


class Package(models.Model):

    file = models.FileField(u'пакет в xml-формате',
                            upload_to='django_banzai/package')
    status = models.CharField(u'статус', choices=STATUS_CODES,
                              max_length=4, blank=True)
    pack_id = models.CharField(u'ID пакета в системе', max_length=100,
                               blank=True)

    emails_all = models.PositiveIntegerField(u'emailов всего')
    emails_correct = models.PositiveIntegerField(u'emailов корректных',
                                                 default=0)

    description = models.CharField(u'описание пакета', max_length=100,
                                   blank=True)

    class Meta:
        verbose_name = u'пакет'
        verbose_name_plural = u'пакеты'

    def __unicode__(self):
        return u'ID: {0}, Статус: "{1}", Описание пакета: "{2}"'.format(
            self.pk,
            self.status,
            self.description
        )


class Report(models.Model):

    package = models.ForeignKey(Package, verbose_name=u'пакет',
                                related_name='reports')
    status = models.CharField(u'статус', choices=STATUS_CODES,
                              max_length=4, blank=True)

    email = models.EmailField(u'email', blank=True)
    reject_code = models.CharField(u'код ошибки', max_length=250,
                                   blank=True)
    reject_message = models.TextField(u'сообщение об ошибке', blank=True)

    class Meta:
        verbose_name = u'отчёт'
        verbose_name_plural = u'отчёты'

    def __unicode__(self):
        return u'Email: {0}, Код ошибки: "{1}"'.format(
            self.email,
            self.reject_code
        )


class ReportFBL(models.Model):

    package = models.ForeignKey(Package, verbose_name=u'пакет',
                                related_name='reports_fbl')
    status = models.CharField(u'статус', choices=STATUS_CODES,
                              max_length=4, blank=True)
    email = models.EmailField(u'email')

    class Meta:
        verbose_name = u'отчёт о FBL'
        verbose_name_plural = u'отчёты о FBL'

    def __unicode__(self):
        return self.email



import os
from setuptools import setup, find_packages


def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()


setup(
    name='django-banzai',
    version='0.1',
    license='MIT',
    description='Django API wrapper for mailing delivery '
                'service: http://get-n-post.ru/',
    long_description=read('README.rst'),
    keywords='mail email send wrapper api delivery django',
    url='https://github.com/saippuakauppias/django-banzai',
    author='Denis Veselov',
    author_email='progr.mail@gmail.com',
    include_package_data=True,
    packages=find_packages(),
    install_requires=['requests', 'lxml', 'django'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP'
    ],
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Package'
        db.create_table('banzai_package', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=4, blank=True)),
            ('pack_id', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('emails_all', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('emails_correct', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
        ))
        db.send_create_signal('banzai', ['Package'])

        # Adding model 'Report'
        db.create_table('banzai_report', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('package', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['banzai.Package'])),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=4, blank=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('reject_code', self.gf('django.db.models.fields.CharField')(max_length=250)),
            ('reject_message', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('banzai', ['Report'])

        # Adding model 'ReportFBL'
        db.create_table('banzai_reportfbl', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('package', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['banzai.Package'])),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=4, blank=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
        ))
        db.send_create_signal('banzai', ['ReportFBL'])


    def backwards(self, orm):
        # Deleting model 'Package'
        db.delete_table('banzai_package')

        # Deleting model 'Report'
        db.delete_table('banzai_report')

        # Deleting model 'ReportFBL'
        db.delete_table('banzai_reportfbl')


    models = {
        'banzai.package': {
            'Meta': {'object_name': 'Package'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'emails_all': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'emails_correct': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pack_id': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '4', 'blank': 'True'})
        },
        'banzai.report': {
            'Meta': {'object_name': 'Report'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'package': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['banzai.Package']"}),
            'reject_code': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'reject_message': ('django.db.models.fields.TextField', [], {}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '4', 'blank': 'True'})
        },
        'banzai.reportfbl': {
            'Meta': {'object_name': 'ReportFBL'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'package': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['banzai.Package']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '4', 'blank': 'True'})
        }
    }

    complete_apps = ['banzai']